/*-
 * Copyright (c) 2001 Lev Walkin <vlm@spelio.net.ru>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ppl-bpf.c,v 1.3 2002/03/22 21:41:36 vlm Exp $
 */

#include "ipcad.h"
#include "opt.h"

#define	bpf_header	((struct bpf_hdr *)packet)

void *
pps_loop(void *psp) {
	packet_source *ps = (packet_source *)psp;
	ssize_t rd = 0;
	char *packet;
	struct pollfd pfd;
	sigset_t set, oset;

	pfd.fd = ps->fd;
	pfd.events = POLLIN;

	sigemptyset(&set);
	sigaddset(&set, SIGALRM);

	while(1) {

		if(signoff_now)
			break;

		if(rd == -1) {
			if(errno != EAGAIN) {
				/* We can't deal with permanent errors.
				 * Just sleep a bit.
				 */
				printf("System call returned %d: %s\n",
					errno, strerror(errno));
				sleep(1);
			}
		}

		sigprocmask(SIG_UNBLOCK, &set, &oset);

		rd = poll(&pfd, 1, -1);

		sigprocmask(SIG_SETMASK, &oset, NULL);

		/* Poll timeout almost impossible thing */
		if(rd == 0) {
			printf("Unexpected timeout on interface %s."
			" Please report to vlm@spelio.net.ru."
			" Sleeping.\n",
			ps->ifname);
			sleep(1);
			continue;
		}

		if((rd == -1) || !(pfd.revents & POLLIN)) {
			rd = -1;
			continue;
		}

		rd = read(ps->fd, ps->buf, ps->bufsize);

		/* Read timeout or failure condition */
		if(rd <= 0)
			continue;
	
		/* Processing packets */
		for(packet = ps->buf; (packet - ps->buf) < rd;
			  packet += BPF_WORDALIGN(bpf_header->bh_hdrlen
				  + bpf_header->bh_caplen) )
			{
			process_packet_data((unsigned char *)ps, bpf_header,
					packet + bpf_header->bh_hdrlen);
			}

	}

	return 0;

}

